import numpy as np
from pystencils import fields
from importlib.resources import files
from pystencilssfg import SourceFileGenerator
from pymatlib.core.yaml_parser import create_alloy_from_yaml
from pymatlib.core.codegen.interpolation_array_container import InterpolationArrayContainer
from pymatlib.core.property_array_extractor import PropertyArrayExtractor


with SourceFileGenerator() as sfg:
    u = fields(f"u: double[2D]", layout='fzyx')

    T_bs = np.array([3243.15, 3248.15, 3258.15, 3278.15], dtype=np.float64)
    E_bs = np.array([1.68e10, 1.69e10, 1.70e10, 1.71e10], dtype=np.float64)

    custom_container = InterpolationArrayContainer("BinarySearchTests", T_bs, E_bs)
    sfg.generate(custom_container)

    T_eq = np.array([3243.15, 3253.15, 3263.15, 3273.15], dtype=np.float64)
    E_neq = np.array([1.68e10, 1.69e10, 1.70e10, 1.71e10], dtype=np.float64)

    custom_container = InterpolationArrayContainer("DoubleLookupTests", np.flip(T_eq), np.flip(E_neq))
    sfg.generate(custom_container)

    yaml_path = files('pymatlib.data.alloys.SS304L').joinpath('SS304L.yaml')
    mat, temp_array = create_alloy_from_yaml(yaml_path, u.center())
    array_extractor = PropertyArrayExtractor(mat, temp_array, u.center)
    arr_container = InterpolationArrayContainer("SS304L", temp_array, array_extractor.energy_density_array)
    sfg.generate(arr_container)
    print(f"extractor.temperature_array: {array_extractor.temperature_array}")
    # print(f"extractor.density_array: {array_extractor.density_array}")
    print(f"extractor.specific_enthalpy_array: {array_extractor.specific_enthalpy_array}")
    print(f"extractor.energy_density_array: {array_extractor.energy_density_array}")
